/*
* This file is part of video-editor-ui
*
* Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies). All rights
* reserved.
*
* Contact: Kristiina Hietanen <Kristiina.Hietanen@nokia.com>
*
* This software, including documentation, is protected by copyright controlled
* by Nokia Corporation. All rights are reserved. Copying, including
* reproducing, storing, adapting or translating, any or all of this material
* requires the prior written consent of Nokia Corporation. This material also
* contains confidential information which may not be disclosed to others
* without the prior written consent of Nokia.
*
*/

import QtQuick 1.1
import com.nokia.meego 1.0

Item {
    id: root
    height: parent.height

    property alias sliderOpacity: backgroundRect.opacity
    property alias minValueLabel: minPosIndicator.text
    property alias positionLabel: valueIndicator.text
    property alias maxValueLabel: maxPosIndicator.text
    property real position: 0.0
    property real minValue: 0.0
    property real maxValue: 1.0
    property real maxTrimSize: 1.0
    property bool dragged: false
    property alias pressed: sliderMouseArea.pressed
    property alias minposPressed: minposArea.pressed
    property alias maxposPressed: maxposArea.pressed

    function updateMouseAreas() {
        var minLeft = minpos.x - 75;
        var minRight = minLeft + 150;
        var sliderLeft = slider.x - 75;
        var sliderRight = sliderLeft + 150;
        var maxLeft = maxpos.x - 75;
        var maxRight = maxLeft + 150;

        /* Don't overlap with the play/pause button */
        if (minLeft < -25) {
            minLeft = -25;
        }

        if (minRight > sliderLeft) {
            minRight = (slider.x + slider.width + minpos.x) / 2;
            sliderLeft = minRight;
        }

        if (maxLeft < sliderRight) {
            sliderRight = (slider.x + slider.width + maxpos.x) / 2;
            maxLeft = sliderRight;
        }

        minposArea.x = minLeft;
        minposArea.width = minRight - minLeft;

        sliderMouseArea.x = sliderLeft;
        sliderMouseArea.width = sliderRight - sliderLeft;

        maxposArea.x = maxLeft;
        maxposArea.width = maxRight - maxLeft;
    }

    onPositionChanged: {
        if (position > 1.0) position = 1.0;
        if (position < 0.0) position = 0.0;
        if (slider.x != position * progressbar.width) {
            slider.x = position * progressbar.width;
        }
        updateMouseAreas();
    }

    onMinValueChanged: {
        if (minValue > 1.0) minValue = 1.0;
        if (minValue < 0.0) minValue = 0.0;
        if (minpos.x != minValue * progressbar.width) {
            minpos.x = minValue * progressbar.width;
        }
        updateMouseAreas();
    }

    onMaxValueChanged: {
        if (maxValue > 1.0) maxValue = 1.0;
        if (maxValue < 0.0) maxValue = 0.0;
        if (maxpos.x != maxValue * progressbar.width) {
            maxpos.x = maxValue * progressbar.width;
        }
        updateMouseAreas();
    }

    onMaxTrimSizeChanged: {
        if (maxValue - minValue > maxTrimSize) {
            maxValue = minValue + maxTrimSize;
        }
    }

    Component.onCompleted: {
        updateMouseAreas();
    }

    Rectangle {
        id: backgroundRect
        color: "black"
        anchors.fill: parent
    }

    Rectangle {
        id: progressbar
        width: parent.width - 40
        anchors.centerIn: parent

        ProgressBar {
            id: leftProgressbar
            anchors.left: parent.left
            anchors.right: minpos.left
            anchors.verticalCenter: parent.verticalCenter
            style: ProgressBarStyle { inverted: true }
        }

        Rectangle {
            id: blueBar
            anchors.left: minpos.left
            anchors.right: slider.horizontalCenter
            y: leftProgressbar.y + 1
            height: leftProgressbar.height - 1
            color: "#6400bc"
        }

        Image {
            anchors.left: minpos.left
            anchors.right: maxpos.left
            y: blueBar.y - 5
            fillMode: Image.TileHorizontally
            source: "dash.png"
            visible: maxTrimSize != 1.0
        }

        Image {
            anchors.left: minpos.left
            anchors.right: maxpos.left
            y: blueBar.y + blueBar.height + 1
            fillMode: Image.TileHorizontally
            source: "dash.png"
            visible: maxTrimSize != 1.0
        }

        ProgressBar {
            anchors.left: slider.horizontalCenter
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            style: ProgressBarStyle { inverted: true }
        }

        MouseArea {
            id: leftMouseArea
            anchors.left: parent.left
            anchors.right: minposArea.left
            anchors.verticalCenter: parent.verticalCenter
            height: root.height
            onPressed: {
                minpos.x = mouseX;
            }
        }

        MouseArea {
            id: centerMouseArea
            anchors.left: minposArea.right
            anchors.right: maxposArea.left
            anchors.verticalCenter: parent.verticalCenter
            height: root.height
            onPressed: {
                root.dragged = true
                slider.x = mouseX + centerMouseArea.x;
            }
            onReleased: {
                root.dragged = false
            }
        }

        MouseArea {
            id: rightMouseArea
            anchors.left: maxposArea.right
            anchors.right: parent.right
            anchors.verticalCenter: parent.verticalCenter
            height: root.height
            onPressed: {
                maxpos.x = mouseX + rightMouseArea.x;
            }
        }

        MouseArea {
            id: sliderMouseArea
            anchors.verticalCenter: slider.verticalCenter
            height: 100
            drag.target: slider; drag.axis: Drag.XAxis
            drag.minimumX: minpos.x
            drag.maximumX: maxpos.x

            onPressed: { root.dragged = true }
            onReleased: { root.dragged = false }
            onClicked: {
                root.dragged = true;
                slider.x = sliderMouseArea.x + mouseX;
                root.dragged = false;
            }
        }

        MouseArea {
            id: minposArea
            anchors.verticalCenter: minpos.verticalCenter
            height: 100
            drag.target: minpos; drag.axis: Drag.XAxis
            drag.minimumX: 0
            drag.maximumX: maxpos.x

            onPressed: { root.dragged = true }
            onReleased: { root.dragged = false }
            onClicked: {
                var pos = minposArea.x + mouseX;
                if (pos < minpos.x) {
                    minpos.x = pos;
                } else {
                    root.dragged = true;
                    slider.x = pos;
                    root.dragged = false;
                }
            }
        }

        MouseArea {
            id: maxposArea
            anchors.verticalCenter: maxpos.verticalCenter
            height: 100
            drag.target: maxpos; drag.axis: Drag.XAxis
            drag.minimumX: minpos.x
            drag.maximumX: progressbar.width

            onPressed: { root.dragged = true }
            onReleased: { root.dragged = false }
            onClicked: {
                var pos = maxposArea.x + mouseX;
                if (pos > maxpos.x) {
                    maxpos.x = pos;
                } else {
                    root.dragged = true;
                    slider.x = pos;
                    root.dragged = false;
                }
            }
        }

        Image {
            id: minpos
            source: "handle.png"
            x: progressbar.width * minValue
            y: -height / 2
            transform: Translate { x: - 2 * minpos.width / 3 }
            onXChanged: {
                minValue = minpos.x / progressbar.width;
                if (position < minValue) {
                    if (!root.dragged) {
                        root.dragged = true;
                        position = minValue;
                        root.dragged = false;
                    } else {
                        position = minValue;
                    }
                }
                if (maxTrimSize > 0.0 && maxValue - minValue > maxTrimSize) {
                    maxValue = minValue + maxTrimSize;
                }
            }
        }

        Image {
            id: maxpos
            source: "handle.png"
            mirror: true
            x: progressbar.width * maxValue
            y: -height / 2
            transform: Translate { x: - maxpos.width / 3 }
            onXChanged: {
                maxValue = maxpos.x / progressbar.width;
                if (position > maxValue) {
                    if (!root.dragged) {
                        root.dragged = true;
                        position = minValue;
                        root.dragged = false;
                    } else {
                        position = minValue;
                    }
                }
                if (maxTrimSize > 0.0 && maxValue - minValue > maxTrimSize) {
                    minValue = maxValue - maxTrimSize;
                }
            }
        }

        Rectangle {
            id: slider
            y: -2
            anchors.verticalCenter: blueBar.verticalCenter
            height: blueBar.height - 2
            width: height
            transform: Translate { x: - slider.width / 2 }
            onXChanged: { position = slider.x / progressbar.width; }
        }

        PositionIndicator {
            id: minPosIndicator
            anchors.bottom: minpos.top
            anchors.horizontalCenter: minpos.horizontalCenter
            visible: minposArea.pressed
            transform: Translate {
                x: - minpos.width / 2 - 5
                y: - 10
            }
            text: getDurationLabel("0:00")
        }

        PositionIndicator {
            id: valueIndicator
            anchors.bottom: slider.top
            anchors.horizontalCenter: slider.horizontalCenter
            visible: sliderMouseArea.pressed
            transform: Translate {
                x: - slider.width / 2
                y: - 25
            }
            text: getDurationLabel("0:00")
        }

        PositionIndicator {
            id: maxPosIndicator
            anchors.bottom: maxpos.top
            anchors.horizontalCenter: maxpos.horizontalCenter
            visible: maxposArea.pressed
            transform: Translate {
                x: - maxpos.width / 2
                y: - 10
            }
            text: getDurationLabel("0:00")
        }
    }
}
