/*
* This file is part of video-editor-ui
*
* Copyright (C) 2011 Nokia Corporation and/or its subsidiary(-ies). All rights
* reserved.
*
* Contact: Kristiina Hietanen <Kristiina.Hietanen@nokia.com>
*
* This software, including documentation, is protected by copyright controlled
* by Nokia Corporation. All rights are reserved. Copying, including
* reproducing, storing, adapting or translating, any or all of this material
* requires the prior written consent of Nokia Corporation. This material also
* contains confidential information which may not be disclosed to others
* without the prior written consent of Nokia.
*
*/

import trimui.videoitem 1.0
import QtQuick 1.0
import com.nokia.meego 1.0
import com.meego.extras 1.0
import QtMobility.systeminfo 1.2
import "UIConstants.js" as UI

TrimUiVideoItem {
    id: page
    width: screen.displayWidth; height: screen.displayHeight

    property string durationLabel : ""
    property alias savedFileName : fileNameField.text
    property bool skipSaveAsSheet : false
    property bool playing : false
    property bool seeking : false
    property alias maxTrimSize : slider.maxTrimSize
    property alias timerInterval: seekbarTimer.interval
    property alias sliderPosition: slider.position
    property alias sliderMinPosition: slider.minValue
    property alias sliderMaxPosition: slider.maxValue
    property alias encodingPercent: encodingProgressBar.value
    property alias selectedAudioFile: audioMenu.selectedAudioFile
    property bool usbMode : false

    property bool __fullscreen : false
    property string __playButtonIcon : UI.PLAY_BUTTON
    property string __audioButtonIcon : UI.AUDIO_BUTTON

    UsbModeBanner {
        /* Show this only when the device is in USB mass-storage mode */
        visible: usbMode
    }

    Connections {
        target: platformWindow

        onActiveChanged: {
            if (!platformWindow.active) {
                forceIdle();
            }
        }
    }

    ScreenSaver {
        id: screenSaver
        screenSaverDelayed: playing
    }

    Timer {
        id: seekbarTimer
        repeat: true
        onTriggered: { slider.position = position(); }
    }

    onPlayingChanged: {
        if (playing) {
            seekbarTimer.start()
            __playButtonIcon = UI.PAUSE_BUTTON
        } else {
            seekbarTimer.stop()
            __playButtonIcon = UI.PLAY_BUTTON
            if ((slider.position >= slider.maxValue ||
                 slider.position <= slider.minValue) && !slider.dragged) {
                seek(slider.minValue)
                seek(-1)
                slider.position = slider.minValue
            }
        }
    }

    function updateLabel() {
        updateDurationLabel(slider.minValue, slider.maxValue)
    }

    function unsetAudio() {
        audioMenu.selectedIndex = audioMenu.original;
    }

    function updateDoneButton() {
        if (slider.minValue == 0.0 && slider.maxValue == 1.0 &&
            audioMenu.selectedIndex == audioMenu.original) {
            doneButton.enabled = false;
            playbutton.enabled = true;
        } else if (slider.minValue == slider.maxValue) {
            doneButton.enabled = false;
            playbutton.enabled = false;
        } else {
            doneButton.enabled = true;
            playbutton.enabled = true;
        }

        if (!playing) {
            __playButtonIcon = playbutton.enabled ? UI.PLAY_BUTTON : UI.PLAY_BUTTON_DIMMED;
        }
    }

    function playPauseClicked() {
        if (durationLabel != "") {
            if (playing) {
                pauseVideo()
            } else {
                playVideo()
            }
        }
    }

    function showBanner(text) {
        infoBanner.text = text;
        infoBanner.show();
    }

    function closeEncodingSheet() {
        encodingSheet.close()
    }

    function closeEventReceived() {
        if (encodingSheet.status == DialogStatus.Closed) {
            Qt.quit();
        } else {
            raise();
            closeConfirmationDialog.open();
        }
    }

    function baseName(str) {
        return str.replace(/.*\//, '');
    }

    Component.onCompleted: {
        theme.colorScheme = 10;
        updateDoneButton();
        screen.setAllowedOrientations(Screen.Landscape);
    }

    MouseArea {
        anchors.top: buttonbar.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: controlbar.top
        onReleased: {
            if (__fullscreen) {
                buttonbar.opacity = 1.0
                controlbar.opacity = 1.0
            } else {
                buttonbar.opacity = 0.0
                controlbar.opacity = 0.0
            }
            __fullscreen = !__fullscreen
        }
    }

    Item {
        anchors.top: buttonbar.bottom
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.bottom: controlbar.top

        Button {
            anchors.centerIn: parent
            height: 60
            width: 60
            iconSource: __playButtonIcon
            visible: !playing && __fullscreen
            onClicked: { playPauseClicked () }
            style: ButtonStyle { inverted: true }
        }

        AudioMenu {
            id: audioMenu
        }
    }

    Item {
        id: controlbar
        anchors.bottom: parent.bottom
        width: parent.width
        height: 68

        MouseArea {
            id: playbutton
            anchors.left: parent.left
            height: parent.height
            width: 100
            onClicked: { playPauseClicked () }

            onPressed: {
                if (playing) {
                    __playButtonIcon = UI.PAUSE_BUTTON_PRESSED;
                } else {
                    __playButtonIcon = UI.PLAY_BUTTON_PRESSED;
                }
            }

            onReleased: {
                if (playing) {
                    __playButtonIcon = UI.PAUSE_BUTTON;
                } else {
                    __playButtonIcon = UI.PLAY_BUTTON;
                }
            }

            Rectangle {
                anchors.fill: parent
                color: "black"
                opacity: UI.CONTROL_BAR_OPACITY
            }

            Image {
                anchors.centerIn: parent
                source: __playButtonIcon
            }
        }

        TrimSlider {
            id: slider
            anchors.left: playbutton.right
            anchors.right: label.left
            sliderOpacity: UI.CONTROL_BAR_OPACITY

            property bool __resumePlayback : false

            onDraggedChanged: {
                if (dragged) {
                    if (playing) {
                        pauseVideo();
                        __resumePlayback = true;
                    }
                    seeking = true;
                    slider.minValueLabel = getDurationLabel(slider.minValue);
                    slider.positionLabel = getDurationLabel(slider.position);
                    slider.maxValueLabel = getDurationLabel(slider.maxValue);
                } else {
                    seeking = false;
                    seek(slider.position);
                    seek(-1)
                    if (__resumePlayback) {
                        playVideo();
                        __resumePlayback = false;
                    }
                }
            }
            onPositionChanged: {
                if (position >= maxValue) {
                    pauseVideo();
                }
                if (slider.dragged || !playing) {
                    if (!slider.maxposPressed && !slider.minposPressed) {
                        seek(slider.position);
                    }
                }
                if (slider.pressed) {
                    slider.positionLabel = getDurationLabel(slider.position)
                }
            }
            onMinValueChanged: {
                updateLabel()
                updateDoneButton()
                slider.minValueLabel = getDurationLabel(slider.minValue)
                if (slider.minposPressed) {
                    seek(slider.minValue);
                }
            }
            onMaxValueChanged: {
                updateLabel()
                updateDoneButton()
                slider.maxValueLabel = getDurationLabel(slider.maxValue)
                if (slider.maxposPressed) {
                    seek(slider.maxValue);
                }
            }
        }

        Rectangle {
            id: label
            anchors.right: audiobutton.left
            height: parent.height
            width: 80
            color: "black"
            opacity: UI.CONTROL_BAR_OPACITY

            Text {
                text: durationLabel
                font.pointSize: 14.0
                color: "white"
                anchors.left: parent.left
                anchors.verticalCenter: parent.verticalCenter
            }

        }

        MouseArea {
            id: audiobutton
            anchors.right: parent.right
            height: parent.height
            width: 100
            onClicked: {
                if (playing) {
                    pauseVideo()
                }
                audioMenu.open();
            }

            Rectangle {
                anchors.fill: parent
                color: "black"
                opacity: UI.CONTROL_BAR_OPACITY
            }

            Image {
                anchors.centerIn: parent
                source: __audioButtonIcon
            }
        }

        Behavior on opacity { PropertyAnimation {} }
    }

    Item {
        id: buttonbar
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.top: parent.top
        height: 64

        Rectangle {
            anchors.fill: parent
            color: "black"
            opacity: UI.CONTROL_BAR_OPACITY
        }

        SheetButton {
            id: doneButton
            //% "!!Done"
            text: qsTrId("qtn_comm_command_done")
            x: parent.width - width - 16
            anchors.verticalCenter: parent.verticalCenter
            property int imgCounter : 0
            onClicked: {
                if (playing) {
                    pauseVideo()
                }
                if (skipSaveAsSheet) {
                    var mute = (audioMenu.selectedIndex == audioMenu.mute);
                    var replace = (audioMenu.selectedIndex == audioMenu.replace);
                    trimVideo (slider.minValue, slider.maxValue, savedFileName, mute, replace);
                } else {
                    thumbnail.source = "image://screenshot/" + imgCounter++;
                    saveAsSheet.open();
                }
            }
            platformStyle: SheetButtonAccentStyle { inverted: true }
        }
        SheetButton {
            //% "!!Cancel"
            text: qsTrId("qtn_comm_cancel")
            x: 16
            anchors.verticalCenter: parent.verticalCenter
            onClicked: { Qt.quit() }
            platformStyle: SheetButtonStyle { inverted: true }
        }

        Behavior on opacity { PropertyAnimation {} }
    }

    Sheet {
        id: saveAsSheet

        //% "!!Save"
        acceptButtonText: qsTrId("qtn_comm_save")
        //% "!!Cancel"
        rejectButtonText: qsTrId("qtn_comm_cancel")

        acceptButton.enabled: fileNameField.acceptableInput

        style: SheetStyle { inverted: true }

        Component.onCompleted: {
            acceptButton.platformStyle.inverted = true;
            rejectButton.platformStyle.inverted = true;
        }

        onAccepted: {
            encodingSheet.open()
        }

        content: Rectangle {
            anchors.fill: parent
            color: "black"

            Column {
                anchors.fill: parent
                anchors.leftMargin: 20
                anchors.rightMargin: 20
                anchors.topMargin: 10
                spacing: 10

                Label {
                    anchors.left: parent.left
                    color: "white"
                    //% "!!Save as"
                    text: qsTrId("qtn_comm_saveas")
                }

                TextField {
                    id: fileNameField
                    width: parent.width
                    text: savedFileName
                    //% "!!Add name"
                    placeholderText: qsTrId("qtn_vied_addname")
                    validator: RegExpValidator {
                        regExp: /[^\.\/][^\/]{0,254}/
                    }
                }

                Label {
                    anchors.left: parent.left
                    color: "white"
                    //% "!!File extension is <B>%1</B>"
                    text: qsTrId("qtn_vied_extension").arg(".mp4")
                }

                Image {
                    id: thumbnail
                    anchors.horizontalCenter: parent.horizontalCenter
                    transform: Translate { y: +20 }
                    width: 360
                    height: 200
                }
            }
        }
    }

    Sheet {
        id: encodingSheet

        acceptButtonText: ""
        //% "!!Cancel"
        rejectButtonText: qsTrId("qtn_comm_cancel")

        style: SheetStyle { inverted: true }

        Component.onCompleted: {
            rejectButton.platformStyle.inverted = true;
        }

        onStatusChanged: {
            if (status == DialogStatus.Open) {
                var mute = (audioMenu.selectedIndex == audioMenu.mute);
                var replace = (audioMenu.selectedIndex == audioMenu.replace);
                if (videoExists(savedFileName)) {
                    savedFileName = defaultOutputFileName();
                    infoBanner.text = qsTrId("qtn_vied_filenameduplicate").arg(savedFileName);
                    infoBanner.show();
                }
                trimVideo (slider.minValue, slider.maxValue, savedFileName, mute, replace);
            }
        }

        onRejected: {
            cancelTrimmingTask();
            encodingPercent = 0;
        }

        content: Rectangle {
            anchors.fill: parent
            color: "black"

            Column {
                anchors.centerIn: parent
                anchors.leftMargin: 20
                anchors.rightMargin: 20
                anchors.topMargin: 10
                width: parent.width
                spacing: 10

                Label {
                    id: encodingLabel
                    anchors.horizontalCenter: parent.horizontalCenter
                    color: "white"
                    //% "!!Encoding %1"
                    text: qsTrId("qtn_vied_encoding").arg(baseName(savedFileName))
                }

                ProgressBar {
                    id: encodingProgressBar
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width - 80
                    minimumValue: 0
                    maximumValue: 100
                    value: 0

                    Timer {
                        id: encodedTimer
                        interval: 2000; running: false
                        onTriggered: { Qt.quit() }
                    }

                    onValueChanged: {
                        if (value == 100) {
                            encodingSheet.rejectButton.text = "";
                            encodingProgressBar.visible = false;
                            //% "!!Encoding completed"
                            encodingLabel.text = qsTrId("qtn_vied_encoding_completed");
                            encodedTimer.start();
                        }
                    }
                }
            }
        }
    }

    QueryDialog {
        id: closeConfirmationDialog
        //% "!!End encoding?"
        message: qsTrId("qtn_vied_end_encoding_query")
        //% "!!Yes"
        acceptButtonText: qsTrId("qtn_comm_command_yes")
        //% "!!No"
        rejectButtonText: qsTrId("qtn_comm_command_no")

        onAccepted: { Qt.quit(); }
    }

    InfoBanner {
        id: infoBanner
    }
}
